package com.atguigu.ssyx.activity.service.impl;

import com.atguigu.ssyx.activity.mapper.ActivityInfoMapper;
import com.atguigu.ssyx.activity.mapper.ActivityRuleMapper;
import com.atguigu.ssyx.activity.mapper.ActivitySkuMapper;
import com.atguigu.ssyx.activity.service.ActivityInfoService;
import com.atguigu.ssyx.activity.service.CouponInfoService;
import com.atguigu.ssyx.activity.service.module.ActivityInfoServiceModule;
import com.atguigu.ssyx.client.product.ProductFeignClient;
import com.atguigu.ssyx.model.activity.ActivityInfo;
import com.atguigu.ssyx.model.activity.ActivityRule;
import com.atguigu.ssyx.model.activity.ActivitySku;
import com.atguigu.ssyx.model.activity.CouponInfo;
import com.atguigu.ssyx.model.order.CartInfo;
import com.atguigu.ssyx.model.product.SkuInfo;
import com.atguigu.ssyx.vo.activity.ActivityRuleVo;
import com.atguigu.ssyx.vo.order.CartInfoVo;
import com.atguigu.ssyx.vo.order.OrderConfirmVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional
public class ActivityInfoServiceImpl extends ServiceImpl<ActivityInfoMapper, ActivityInfo> implements ActivityInfoService {
    @Autowired
    private ProductFeignClient productFeignClient;
    @Autowired
    private ActivityRuleMapper activityRuleMapper;
    @Autowired
    private ActivitySkuMapper activitySkuMapper;
    @Autowired
    private CouponInfoService couponInfoService;
    @Autowired
    private ActivityInfoServiceModule module;

    /**
     * 获取分页列表
     *
     * @param pageParam 分页查询条件
     * @return 分页结果
     */
    @Override
    public IPage<ActivityInfo> selectPage(Page<ActivityInfo> pageParam) {
        QueryWrapper<ActivityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("id");
        IPage<ActivityInfo> page = baseMapper.selectPage(pageParam, queryWrapper);

        page.getRecords().forEach(item -> {
            item.setActivityTypeString(item.getActivityType().getComment());
        });
        return page;
    }

    /**
     * 获取活动规则
     *
     * @param activityId 当前id
     * @return 返回集合
     */
    @Override
    public Map<String, Object> findActivityRuleList(Long activityId) {
        Map<String, Object> result = new HashMap<>();

        // 活动信息列表
        List<ActivityRule> activityInfoList = activityRuleMapper.selectList(
                Wrappers.<ActivityRule>lambdaQuery().eq(ActivityRule::getActivityId, activityId)
        );
        result.put("activityRuleList", activityInfoList);

        // 查询活动列表
        List<ActivitySku> activitySkuList = activitySkuMapper.selectList(
                Wrappers.<ActivitySku>lambdaQuery().eq(ActivitySku::getActivityId, activityId)
        );
        List<Long> skuIdList = activitySkuList.stream().map(ActivitySku::getSkuId).collect(Collectors.toList());

        // 通过远程调用根据skuId得到商品信息
        List<SkuInfo> skuInfoList = productFeignClient.findSkuInfoList(skuIdList).getData();
        result.put("skuInfoList", skuInfoList);
        return result;
    }

    /**
     * 新增活动规则
     *
     * @param vo 活动规则
     */
    @Override
    public void saveActivityRule(ActivityRuleVo vo) {
        Long activityId = vo.getActivityId();
        List<ActivityRule> activityRuleList = vo.getActivityRuleList();
        List<ActivitySku> activitySkuList = vo.getActivitySkuList();

        // 根据id查询活动信息
        ActivityInfo activityInfo = getById(activityId);

        // 删除之前规则数据
        activityRuleMapper.delete(Wrappers.<ActivityRule>lambdaQuery().eq(ActivityRule::getActivityId, activityId));
        activitySkuMapper.delete(Wrappers.<ActivitySku>lambdaQuery().eq(ActivitySku::getActivityId, activityId));

        // 获取规则列表数据
        activityRuleList.forEach(activityRule -> {
            activityRule.setActivityId(activityId);
            activityRule.setActivityType(activityInfo.getActivityType());
            activityRuleMapper.insert(activityRule);
        });

        // 获取规则范围数据
        activitySkuList.forEach(activitySku -> {
            activitySku.setActivityId(activityId);
            activitySkuMapper.insert(activitySku);
        });
    }

    /**
     * 根据关键字获取sku列表，活动使用
     *
     * @param keyword 关键字
     * @return 活动列表
     */
    @Override
    public List<SkuInfo> findSkuInfoByKeyword(String keyword) {
        // 根据关键字查询sku匹配内容
        List<SkuInfo> skuInfoList = productFeignClient.findSkuInfoByKeyword(keyword).getData();

        if (!skuInfoList.isEmpty()) {
            List<Long> skuIdList = skuInfoList.stream().map(SkuInfo::getId).collect(Collectors.toList());

            // 判断添加商品之前是否参加过活动
            List<Long> existSkuIdList = baseMapper.selectSkuIdListExist(skuIdList);
            // 如果参加的话就剔除
            skuInfoList.removeIf(skuInfo -> existSkuIdList.contains(skuInfo.getId()));

            // 返回结果
            return skuInfoList;
        }
        return skuInfoList;
    }

    /**
     * * 根据skuId列表获取促销信息
     *
     * @param skuIdList 列表
     * @return Map<Long, List < String>>
     */
    @Override
    public Map<Long, List<String>> findActivity(List<Long> skuIdList) {
        Map<Long, List<String>> result = new HashMap<>();
        // skuIdList遍历，得到每个skuId
        skuIdList.forEach(skuId -> {
            // 根据skuId进行查询，查询sku对应活动里面规则列表
            List<ActivityRule> activityRuleList =
                    baseMapper.findActivityRule(skuId);
            // 数据封装，规则名称
            if (!CollectionUtils.isEmpty(activityRuleList)) {
                List<String> ruleList = new ArrayList<>();
                // 把规则名称处理
                for (ActivityRule activityRule : activityRuleList) {
                    ruleList.add(module.getRuleDesc(activityRule));
                }
                result.put(skuId, ruleList);
            }
        });
        return result;
    }

    /**
     * * 根据skuId获取促销与优惠券信息
     *
     * @param skuId  商品id
     * @param userId 用户id
     * @return 营销的活动
     */
    @Override
    public Map<String, Object> findActivityAndCoupon(Long skuId, Long userId) {
        // 一个sku只能有一个促销活动，一个活动有多个活动规则（如满赠，满100送10，满500送50）
        List<ActivityRule> activityRuleList = module.findActivityRuleList(skuId);
        // 获取优惠券信息
        List<CouponInfo> couponInfoList = couponInfoService.findCouponInfo(skuId, userId);
        Map<String, Object> map = new HashMap<>();
        map.put("activityRuleList", activityRuleList);
        map.put("couponInfoList", couponInfoList);
        return map;
    }

    /**
     * * 获取购物车满足条件的促销与优惠券信息
     *
     * @param cartInfoList 购物车信息列表
     * @param userId       用户ID
     * @return OrderConfirmVo
     */
    @Override
    public OrderConfirmVo findCartActivityAndCoupon(List<CartInfo> cartInfoList, Long userId) {
        // 获取购物车，每个购物车参与活动，根据活动规则分组
        List<CartInfoVo> carInfoVoList = module.findCartActivityList(cartInfoList);
        // 促销活动优惠的总金额
        BigDecimal activityReduceAmount = carInfoVoList.stream()
                .filter(carInfoVo -> null != carInfoVo.getActivityRule())
                .map(carInfoVo -> carInfoVo.getActivityRule().getReduceAmount())
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 购物车可使用的优惠券列表
        List<CouponInfo> couponInfoList = couponInfoService.findCartCouponInfo(cartInfoList, userId);
        // 优惠券可优惠的总金额，一次购物只能使用一张优惠券
        BigDecimal couponReduceAmount = new BigDecimal(0);
        if (!CollectionUtils.isEmpty(couponInfoList)) {
            couponReduceAmount = couponInfoList.stream()
                    .filter(couponInfo -> couponInfo.getIsOptimal().intValue() == 1)
                    .map(couponInfo -> couponInfo.getAmount())
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }

        // 购物车总金额
        BigDecimal carInfoTotalAmount = cartInfoList.stream()
                .filter(cartInfo -> cartInfo.getIsChecked() == 1)
                .map(cartInfo -> cartInfo.getCartPrice().multiply(new BigDecimal(cartInfo.getSkuNum())))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 购物车原始总金额
        BigDecimal originalTotalAmount = cartInfoList.stream()
                .filter(cartInfo -> cartInfo.getIsChecked() == 1)
                .map(cartInfo -> cartInfo.getCartPrice().multiply(new BigDecimal(cartInfo.getSkuNum())))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 最终总金额
        BigDecimal totalAmount = originalTotalAmount.subtract(activityReduceAmount).subtract(couponReduceAmount);

        OrderConfirmVo orderTradeVo = new OrderConfirmVo();
        orderTradeVo.setCarInfoVoList(carInfoVoList);
        orderTradeVo.setActivityReduceAmount(activityReduceAmount);
        orderTradeVo.setCouponInfoList(couponInfoList);
        orderTradeVo.setCouponReduceAmount(couponReduceAmount);
        orderTradeVo.setOriginalTotalAmount(originalTotalAmount);
        orderTradeVo.setTotalAmount(totalAmount);
        return orderTradeVo;
    }
}
