package com.atguigu.ssyx.cart.service.impl;

import com.atguigu.ssyx.cart.service.CartInfoService;
import com.atguigu.ssyx.cart.service.module.CartInfoServiceModule;
import com.atguigu.ssyx.client.product.ProductFeignClient;
import com.atguigu.ssyx.common.constant.MessageConstant;
import com.atguigu.ssyx.common.exception.BunnyException;
import com.atguigu.ssyx.common.result.ResultCodeEnum;
import com.atguigu.ssyx.enums.SkuType;
import com.atguigu.ssyx.model.base.BaseEntity;
import com.atguigu.ssyx.model.order.CartInfo;
import com.atguigu.ssyx.model.product.SkuInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.BoundHashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class CartInfoServiceImpl implements CartInfoService {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private CartInfoServiceModule module;
    @Autowired
    private ProductFeignClient productFeignClient;

    /**
     * * 添加购物车
     *
     * @param skuId  商品id
     * @param userId 用户id
     * @param skuNum 商品数量
     */
    @Override
    public void addToCart(Long skuId, Long userId, Integer skuNum) {
        CartInfo cartInfo = null;
        // 因为购物车数据存储在REdis中
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String, CartInfo> hashOperations = redisTemplate.boundHashOps(cartKey);
        // 根据第一步查询的结果，得到skuId+skuNum关系
        if (Boolean.TRUE.equals(hashOperations.hasKey(skuId))) {
            cartInfo = hashOperations.get(skuId);
            int currentSkuNum = cartInfo.getSkuNum() + skuNum;
            if (currentSkuNum <= 1) throw new BunnyException(MessageConstant.SKU_NUM_CANNOT_BE_LESS);

            // 更新cartInfo对象
            cartInfo.setSkuNum(currentSkuNum);
            cartInfo.setCurrentBuyNum(currentSkuNum);
            // 判断商品不能大于限购数量
            if (currentSkuNum > cartInfo.getPerLimit()) throw new BunnyException(ResultCodeEnum.SKU_LIMIT_ERROR);
            // 更新其他值，加入购物车商品默认选中
            cartInfo.setIsChecked(1);
            cartInfo.setUpdateTime(new Date());
        } else {
            skuNum = 1;
            cartInfo = new CartInfo();
            SkuInfo skuInfo = productFeignClient.getSkuInfo(skuId).getData();
            if (skuInfo == null) {
                throw new BunnyException(ResultCodeEnum.DATA_ERROR);
            }
            cartInfo.setSkuId(skuId);
            cartInfo.setCategoryId(skuInfo.getCategoryId());
            cartInfo.setSkuType(skuInfo.getSkuType());
            cartInfo.setIsNewPerson(skuInfo.getIsNewPerson());
            cartInfo.setUserId(userId);
            cartInfo.setCartPrice(skuInfo.getPrice());
            cartInfo.setSkuNum(skuNum);
            cartInfo.setCurrentBuyNum(skuNum);
            cartInfo.setSkuType(SkuType.COMMON.getCode());
            cartInfo.setPerLimit(skuInfo.getPerLimit());
            cartInfo.setImgUrl(skuInfo.getImgUrl());
            cartInfo.setSkuName(skuInfo.getSkuName());
            cartInfo.setWareId(skuInfo.getWareId());
            cartInfo.setIsChecked(1);
            cartInfo.setStatus(1);
            cartInfo.setCreateTime(new Date());
            cartInfo.setUpdateTime(new Date());
        }

        hashOperations.put(skuId.toString(), cartInfo);
        module.setCartKeyExpire(cartKey);
    }

    /**
     * * 删除购物车内容
     *
     * @param skuId  商品id
     * @param userId 用户id
     */
    @Override
    public void deleteCart(Long skuId, Long userId) {
        BoundHashOperations<String, String, CartInfo> hashOperations = redisTemplate.boundHashOps(module.getCartKey(skuId));
        if (Boolean.TRUE.equals(hashOperations.hasKey(skuId.toString()))) {
            hashOperations.delete(skuId.toString());
        }
    }

    /**
     * * 清空购物车
     *
     * @param userId 用户id
     */
    @Override
    public void deleteAllCart(Long userId) {
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String, CartInfo> hashOperations = redisTemplate.boundHashOps(cartKey);
        List<CartInfo> cartInfoList = hashOperations.values();
        // cartInfoList 不为空
        if (cartInfoList != null) {
            cartInfoList.forEach(cartInfo -> {
                Long skuId = cartInfo.getSkuId();
                hashOperations.delete(skuId);
            });
        }
    }

    /**
     * * 批量删除购物车
     *
     * @param userId    用户id
     * @param skuIdList 商品的id
     */
    @Override
    public void batchDeleteCart(Long userId, List<Long> skuIdList) {
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String, CartInfo> hashOperations = redisTemplate.boundHashOps(cartKey);
        // 如果购物车中列表不为空
        if (skuIdList != null) {
            skuIdList.forEach(skuId -> hashOperations.delete(skuId.toString()));
        }
    }

    /**
     * * 查询购物车列表
     *
     * @param userId 用户id
     * @return 购物车信息列表
     */
    @Override
    public List<CartInfo> getCartList(Long userId) {
        List<CartInfo> cartInfoList = new ArrayList<>();
        // 用户ID不存在返回空数组
        if (StringUtils.isEmpty(userId.toString())) {
            return cartInfoList;
        }
        // 查询Redis之后做排序
        BoundHashOperations<String, String, CartInfo> hashOperations = redisTemplate.boundHashOps(module.getCartKey(userId));
        cartInfoList = hashOperations.values();
        // 当购物车数据不为空时，做降序排列
        if (cartInfoList != null) {
            cartInfoList.sort(Comparator.comparing(BaseEntity::getCreateTime));
        }
        // 返回结果
        return cartInfoList;
    }

    /**
     * * 更新选中状态
     *
     * @param userId    用户id
     * @param isChecked 是否选中
     * @param skuId     商品Id
     */
    @Override
    public void checkCart(Long userId, Integer isChecked, Long skuId) {
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String, CartInfo> boundedHashOps = redisTemplate.boundHashOps(cartKey);
        CartInfo cartInfo = boundedHashOps.get(skuId.toString());
        if (cartInfo != null) {
            cartInfo.setIsChecked(isChecked);
            boundedHashOps.put(skuId.toString(), cartInfo);
            // 设置key过期时间
            module.setCartKeyExpire(cartKey);
        }
    }

    /**
     * * 更新全部选中
     *
     * @param userId    用户id
     * @param isChecked 是否选中
     */
    @Override
    public void checkAllCart(Long userId, Integer isChecked) {
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String, CartInfo> boundHashOps = redisTemplate.boundHashOps(cartKey);
        List<CartInfo> cartInfoList = boundHashOps.values();

        // 当列表不能为空时做处理
        if (cartInfoList != null) {
            cartInfoList.forEach(cartInfo -> {
                cartInfo.setIsChecked(isChecked);
                boundHashOps.put(cartInfo.getSkuId().toString(), cartInfo);
            });
            // 设置过期时间
            module.setCartKeyExpire(cartKey);
        }
    }

    /**
     * * 批量选择购物车
     *
     * @param skuIdList 商品的id列表
     * @param userId    用户id
     * @param isChecked 是否选中
     */
    @Override
    public void batchCheckCart(List<Long> skuIdList, Long userId, Integer isChecked) {
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String, CartInfo> boundHashOps = redisTemplate.boundHashOps(cartKey);

        // 边路skuId将值更新
        skuIdList.forEach(skuId -> {
            CartInfo cartInfo = boundHashOps.get(skuId.toString());
            if (cartInfo != null) {
                cartInfo.setIsChecked(isChecked);
            }
        });
    }

    /**
     * * 根据用户Id查询购物车列表
     *
     * @param userId 用户ID
     * @return 购物车列表
     */
    @Override
    public List<CartInfo> getCartCheckedList(Long userId) {
        String cartKey = module.getCartKey(userId);
        BoundHashOperations<String, String , CartInfo> hashOperations = redisTemplate.boundHashOps(cartKey);
        List<CartInfo> cartInfoList = hashOperations.values();
        // 当购物车为空时抛出异常
        if (cartInfoList==null) {
            throw new BunnyException(MessageConstant.CART_IS_EMPTY);
        }

        cartInfoList = cartInfoList.stream().filter(cartInfo -> cartInfo.getIsChecked() == 1).collect(Collectors.toList());
        return cartInfoList;
    }
}
